<?php

namespace Service;

class NutritionalReport {

    protected $entityManager;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager){
        $this->entityManager = $entityManager;
    }

    public function getFamilies($CDFILIAL){
        $params = array(
            ':CDFILIAL' => $CDFILIAL
        );

        $familyData = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_FAMILY_DATA, $params);

        if(empty($familyData)){
            throw new \Exception("Nenhuma família de saldo encontrada para a filial ". $CDFILIAL);
        }

        return $familyData;
    }

    public function getSalesman($CDFILIAL){
        $params = array(
            ':CDFILIAL' => $CDFILIAL
        );

        $CDVENDEDOR = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_SALESMAN_FILI, $params);

        if(!$CDVENDEDOR['CDVENDEDOR']){
            $CDVENDEDOR = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_SALESMAN_GENERAL);
        }

        if(!$CDVENDEDOR['CDVENDEDOR']){
            throw new \Exception("Vendedor padrão não encontrado.");
        }

        return $CDVENDEDOR;
    }

    public function getRegister($CDVENDEDOR){
        $params = array(
            ':CDVENDEDOR' => $CDVENDEDOR
        );

        $CDCAIXA = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_REGISTER, $params);

        if(!$CDCAIXA['CDCAIXA']){
            throw new \Exception("Caixa não encontrado.");
        }

        return $CDCAIXA;
    }

    public function getGroupProd($CDFILIAL, $CDFAMILISALD){
        $params = array (
            ':CDFILIAL' => $CDFILIAL,
            ':CDFAMILISALD' => $CDFAMILISALD
        );

        $GRUPOPRODUTO = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_FAM_GROUP_PRODUCT, $params);

        if(!$GRUPOPRODUTO['CDGRUPPROD']){
            unset($params[':CDFAMILISALD']);
            $GRUPOPRODUTO = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_FILI_GROUP_PRODUCT, $params);
            if(!$GRUPOPRODUTO['CDGRUPPROD']){
                $GRUPOPRODUTO = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_GER_GROUP_PRODUCT);
            }
        }

        if(!$GRUPOPRODUTO['CDGRUPPROD']){
            throw new \Exception("Grupo de produtos não encontrado.");
        }

        return $GRUPOPRODUTO['CDGRUPPROD'];
    }

    public function getProducts($CDFILIAL, $CDCAIXA, $CDGRUPPROD){
        $params = array(
            ':CDCAIXA'  => $CDCAIXA,
            ':CDFILIAL' => $CDFILIAL,
            ':CDGRUPPROD' => $CDGRUPPROD
        );

        $products = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCTS_NUTRITIONAL, $params);

        if(empty($products)){
            $arr = array(
                'error' => true,
                'message' => "Produtos não encontrados para a filial " . $CDFILIAL ."."
            );
        } else {
            $arr = array(
                'error' => false,
                'products' => $products
            );
        }

        return $arr;
    }

    public function getProductsSold($CDFILIAL, $CDFAMILISALD, $CDGRUPPROD, $DTINI, $DTFIM){
        $params = array(
            ':CDFAMILISALD'  => $CDFAMILISALD,
            ':CDFILIAL' => $CDFILIAL,
            ':CDGRUPPROD' => $CDGRUPPROD,
            ':DTINI' => $DTINI,
            ':DTFIM' => $DTFIM
        );

        $productsSold = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCTS_SOLD, $params);

        if(empty($productsSold)){
            $arr = array(
                'error' => true,
                'message' => "Não há produtos vendidos no período " . $DTINI ." - " . $DTFIM . ' para a filial ' . $CDFILIAL . ', familia ' . $CDFAMILISALD
            );
        } else {
            $arr = array(
                'error' => false,
                'productsSold' => $productsSold
            );
        }

        return $arr;
    }

    public function getUserShoppingData($CDFILIAL, $CDFAMILISALD, $CDCLIENTE, $CDCONSUMIDOR, $CDGRUPPROD, $DTINI, $DTFIM){
        $params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':CDFAMILISALD' => $CDFAMILISALD,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDGRUPPROD' => $CDGRUPPROD,
            ':DTINI' => $DTINI,
            ':DTFIM' => $DTFIM
        );

        $shoppingData = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_SHOPPING_DATA, $params);

        if(empty($shoppingData)){
            $arr = array(
                'error' => true,
                'message' => 'Não há produtos vendidos para esse usuário no período ' . $DTINI . ' - ' . $DTFIM
            );
        } else {
            $arr = array(
                'error' => false,
                'shoppingData' => $shoppingData
            );
        }

        return $arr;
    }
}