<?php
namespace Zeedhi\PaymentPlatform\Implementations\MundiPagg;

use Zeedhi\PaymentPlatform\CreditCard;
use Zeedhi\PaymentPlatform\NonInstantBuy\NonInstantBuyTransaction;
use Zeedhi\PaymentPlatform\InstantBuy\InstantBuyTransaction;
use Zeedhi\PaymentPlatform\BankSlip\BankSlipTransaction;
use Zeedhi\PaymentPlatform\Revoke\Revoke;
use Zeedhi\PaymentPlatform\Consult\Consult;
use Zeedhi\PaymentPlatform\RequestURL;
use Zeedhi\PaymentPlatform\Address;
use Zeedhi\PaymentPlatform\Customer;
use Zeedhi\PaymentPlatform\Exception;

class PaymentImplTest extends \PHPUnit_Framework_TestCase {

	protected $paymentImplMundiPagg;
	protected $client;

	public function setUp(){
		$this->httpRequest = $this->getMockBuilder(HttpRequest::class)
							 ->setMethods(array('setUrl', 'setOption', 'execute', 'getInfo', 'close'))
            				 ->disableOriginalConstructor()
            				 ->getMock();

		$this->paymentImplMundiPagg = new PaymentImpl($this->client, $this->httpRequest);
	}

	public function testCreateToken(){
		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Aura");
		$customer = new Customer();
        $customer->setId("cus_123234");

        $jsonResponse = '{
        	"id": "card_1233454"
        }';

		$response = array(
            "InstantBuyKey" => "card_1233454",
            "CustomerId" => "cus_123234"
        );

		$HTTPStatusCode = '200';

		$this->httpRequest->expects($this->once())
        			 ->method('setUrl');

        $this->httpRequest->expects($this->exactly(4))
        			 ->method('setOption');

		$this->httpRequest->expects($this->once())
        			 ->method('execute')
        			 ->willReturn($jsonResponse);

        $this->httpRequest->expects($this->once())
        			 ->method('getInfo')
        			 ->willReturn($HTTPStatusCode);

        $this->httpRequest->expects($this->once())
        			 ->method('close');

        $theResponse = $this->paymentImplMundiPagg->createToken($creditCard, $customer);

        $this->assertEquals($theResponse, $response);
	}

	public function testCreateTokenWithOutCustomerId(){
		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Aura");
		$customer = new Customer();
        $customer->setName("Stephen Vincent Strange");
        $customer->setEmail("dr-stranger@avengers.com");
        $customer->setDocument("12313321");
        $customer->setType(Customer::INDIVIDUAL);
        $customer->setPhone("999645466");
        $address = new Address("Rua Teste", "456", "Casa", "bairro", "city", "ST", "123123123", "BR");
        $customer->setAddress($address);

        $jsonResponseCustomer = '{
        	"id": "cus_123234"
        }';

        $jsonResponse = '{
        	"id": "card_1233454"
        }';

		$response = array(
            "InstantBuyKey" => "card_1233454",
            "CustomerId" => "cus_123234"
        );

		$HTTPStatusCode = '200';

		$this->httpRequest->expects($this->exactly(2))
        			 ->method('setUrl');

        $this->httpRequest->expects($this->exactly(8))
        			 ->method('setOption');

		$this->httpRequest->expects($this->exactly(2))
        			 ->method('execute')
        			 ->willReturn($jsonResponseCustomer, $jsonResponse);

        $this->httpRequest->expects($this->exactly(2))
        			 ->method('getInfo')
        			 ->willReturn($HTTPStatusCode);

        $this->httpRequest->expects($this->exactly(2))
        			 ->method('close');

        $theResponse = $this->paymentImplMundiPagg->createToken($creditCard, $customer);

        $this->assertEquals($theResponse, $response);
	}

	public function testCreateTokenWithCustomerTypeCompany(){
		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Aura");
		$customer = new Customer();
        $customer->setName("Stephen Vincent Strange");
        $customer->setEmail("dr-stranger@avengers.com");
        $customer->setDocument("12313321");
        $customer->setType(Customer::COMPANY);
        $customer->setPhone("999645466");
        $address = new Address("Rua Teste", "456", "Casa", "bairro", "city", "ST", "123123123", "BR");
        $customer->setAddress($address);

        $jsonResponseCustomer = '{
        	"id": "cus_123234"
        }';

        $jsonResponse = '{
        	"id": "card_1233454"
        }';

		$response = array(
            "InstantBuyKey" => "card_1233454",
            "CustomerId" => "cus_123234"
        );

		$HTTPStatusCode = '200';

		$this->httpRequest->expects($this->exactly(2))
        			 ->method('setUrl');

        $this->httpRequest->expects($this->exactly(8))
        			 ->method('setOption');

		$this->httpRequest->expects($this->exactly(2))
        			 ->method('execute')
        			 ->willReturn($jsonResponseCustomer, $jsonResponse);

        $this->httpRequest->expects($this->exactly(2))
        			 ->method('getInfo')
        			 ->willReturn($HTTPStatusCode);

        $this->httpRequest->expects($this->exactly(2))
        			 ->method('close');

        $theResponse = $this->paymentImplMundiPagg->createToken($creditCard, $customer);

        $this->assertEquals($theResponse, $response);
	}

	public function testCreateTokenWithCustomerTypeError(){
		$this->expectException(Exception::class);
        $this->expectExceptionMessage('Unsupported Customer Type.');
        $this->expectExceptionCode(Exception::UNSUPPORTED_CUSTOMER_TYPE);

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Aura");
		$customer = new Customer();
		$customer->setDocument("12313321");
        $customer->setType("Error");

        $this->paymentImplMundiPagg->createToken($creditCard, $customer);
	}

	public function testCreateTokenWithHTTPStatusCodeError(){
		$this->expectException(Exception::class);
        $this->expectExceptionMessage("HTTP Status: 400. API MundiPagg informa o seguinte erro: Error");
        $this->expectExceptionCode(Exception::API_MUNDIPAGG_ERROR);
		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Aura");
		$customer = new Customer();
        $customer->setId("cus_123234");

        $jsonResponse = '{
        	"message": "Error"
        }';

		$HTTPStatusCode = '400';

		$this->httpRequest->expects($this->once())
        			 ->method('setUrl');

        $this->httpRequest->expects($this->exactly(4))
        			 ->method('setOption');

		$this->httpRequest->expects($this->once())
        			 ->method('execute')
        			 ->willReturn($jsonResponse);

        $this->httpRequest->expects($this->once())
        			 ->method('getInfo')
        			 ->willReturn($HTTPStatusCode);

        $this->httpRequest->expects($this->once())
        			 ->method('close');

        $this->paymentImplMundiPagg->createToken($creditCard, $customer);
	}
	public function testInstantBuyTransaction(){

		$customer = new Customer();
        $customer->setId("cus_123234");
		$instantBuyTransaction = new InstantBuyTransaction("100.00", "0000001", "000000001-000002", $customer);

        $jsonResponse = '{
        	"id": "ch_1233454",
        	"customer": {
        		"id": "cus_123234"
        	}
        }';

		$response = "ch_1233454";

		$HTTPStatusCode = '200';

		$this->httpRequest->expects($this->once())
        			 ->method('setUrl');

        $this->httpRequest->expects($this->exactly(4))
        			 ->method('setOption');

		$this->httpRequest->expects($this->once())
        			 ->method('execute')
        			 ->willReturn($jsonResponse);

        $this->httpRequest->expects($this->once())
        			 ->method('getInfo')
        			 ->willReturn($HTTPStatusCode);

        $this->httpRequest->expects($this->once())
        			 ->method('close');

        $theResponse = $this->paymentImplMundiPagg->instantBuyTransaction($instantBuyTransaction);

        $this->assertEquals($theResponse, $response);
	}

	public function testInstantBuyTransactionWithOutCustomerId(){

		$customer = new Customer();
        $customer->setName("Stephen Vincent Strange");
        $customer->setEmail("dr-stranger@avengers.com");
        $customer->setDocument("12313321");
        $customer->setType(Customer::INDIVIDUAL);
        $customer->setPhone("999645466");
        $address = new Address("Rua Teste", "456", "Casa", "bairro", "city", "ST", "123123123", "BR");
        $customer->setAddress($address);

		$instantBuyTransaction = new InstantBuyTransaction("100.00", "0000001", "000000001-000002", $customer);

        $jsonResponse = '{
        	"id": "ch_1233454",
        	"customer": {
        		"id": "cus_31231231"
        	}
        }';

		$response = "ch_1233454";

		$HTTPStatusCode = '200';

		$this->httpRequest->expects($this->once())
        			 ->method('setUrl');

        $this->httpRequest->expects($this->exactly(4))
        			 ->method('setOption');

		$this->httpRequest->expects($this->once())
        			 ->method('execute')
        			 ->willReturn($jsonResponse);

        $this->httpRequest->expects($this->once())
        			 ->method('getInfo')
        			 ->willReturn($HTTPStatusCode);

        $this->httpRequest->expects($this->once())
        			 ->method('close');

        $theResponse = $this->paymentImplMundiPagg->instantBuyTransaction($instantBuyTransaction);

        $this->assertEquals($theResponse, $response);
	}

	public function testNonInstantBuyTransaction(){

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Visa");

		$customer = new Customer();
        $customer->setId("cus_123234");

		$nonInstantBuyTransaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);
        $jsonResponse = '{
        	"id": "ch_1233454",
        	"customer": {
        		"id": "cus_123234"
        	}
        }';

		$response = "ch_1233454";

		$HTTPStatusCode = '200';

		$this->httpRequest->expects($this->once())
        			 ->method('setUrl');

        $this->httpRequest->expects($this->exactly(4))
        			 ->method('setOption');

		$this->httpRequest->expects($this->once())
        			 ->method('execute')
        			 ->willReturn($jsonResponse);

        $this->httpRequest->expects($this->once())
        			 ->method('getInfo')
        			 ->willReturn($HTTPStatusCode);

        $this->httpRequest->expects($this->once())
        			 ->method('close');

        $theResponse = $this->paymentImplMundiPagg->nonInstantBuyTransaction($nonInstantBuyTransaction);

        $this->assertEquals($theResponse, $response);
	}

	public function testNonInstantBuyTransactionWithOutCustomerId(){

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Visa");

		$customer = new Customer();
        $customer->setName("Stephen Vincent Strange");
        $customer->setEmail("dr-stranger@avengers.com");
        $customer->setDocument("12313321");
        $customer->setType(Customer::INDIVIDUAL);
        $customer->setPhone("999645466");
        $address = new Address("Rua Teste", "456", "Casa", "bairro", "city", "ST", "123123123", "BR");
        $customer->setAddress($address);

		$nonInstantBuyTransaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

        $jsonResponse = '{
        	"id": "ch_1233454",
        	"customer": {
        		"id": "cus_31231231"
        	}
        }';

		$response = "ch_1233454";

		$HTTPStatusCode = '200';

		$this->httpRequest->expects($this->once())
        			 ->method('setUrl');

        $this->httpRequest->expects($this->exactly(4))
        			 ->method('setOption');

		$this->httpRequest->expects($this->once())
        			 ->method('execute')
        			 ->willReturn($jsonResponse);

        $this->httpRequest->expects($this->once())
        			 ->method('getInfo')
        			 ->willReturn($HTTPStatusCode);

        $this->httpRequest->expects($this->once())
        			 ->method('close');

        $theResponse = $this->paymentImplMundiPagg->nonInstantBuyTransaction($nonInstantBuyTransaction);

        $this->assertEquals($theResponse, $response);
	}

	public function testBankBBWithOutCustomerId(){

		$street = "Rua Pernambuco";
		$number = "1000";
		$complement = "Cj 102";
		$district = "Funcionários";
		$city = "Belo Horizonte";
		$state = "Ms";
		$zipCode = "30130-151";
		$country = "BR";
		$clientAddress = new Address($street, $number, $complement, $district, $city, $state, $zipCode, $country);


		$customer = new Customer();
        $customer->setName("Stephen Vincent Strange");
        $customer->setEmail("dr-stranger@avengers.com");
        $customer->setDocument("12313321");
        $customer->setType(Customer::INDIVIDUAL);
        $customer->setPhone("999645466");
        $address = new Address("Rua Teste", "456", "Casa", "bairro", "city", "ST", "123123123", "BR");
        $customer->setAddress($address);

		$bank = "001";
		$value = "100.25";
		$documentNumber = "00000001";
		$transactionId = "0000001";
		$instructions = "Boleto de testes Teknisa";
		$daysToAddInBankSlipExpirationDate = "5";
		$bankSlipTransaction = new BankSlipTransaction($bank, $value, $documentNumber, $transactionId, $instructions, $daysToAddInBankSlipExpirationDate, $customer);

        $jsonResponse = '{
        	"id": "ch_1233454",
        	"last_transaction": {
        		"url": "https://urlteste.com"
        	},
        	"customer": {
        		"id": "cus_31231231"
        	}
        }';

		$response = array(
			"TransactionKey" => "ch_1233454",
			"BoletoUrl" => "https://urlteste.com",
		);

		$HTTPStatusCode = '200';

		$this->httpRequest->expects($this->once())
        			 ->method('setUrl');

        $this->httpRequest->expects($this->exactly(4))
        			 ->method('setOption');

		$this->httpRequest->expects($this->once())
        			 ->method('execute')
        			 ->willReturn($jsonResponse);

        $this->httpRequest->expects($this->once())
        			 ->method('getInfo')
        			 ->willReturn($HTTPStatusCode);

        $this->httpRequest->expects($this->once())
        			 ->method('close');

        $theResponse = $this->paymentImplMundiPagg->bankSlipTransaction($bankSlipTransaction);

        $this->assertEquals($theResponse, $response);
	}

	public function testBankBB(){

		$street = "Rua Pernambuco";
		$number = "1000";
		$complement = "Cj 102";
		$district = "Funcionários";
		$city = "Belo Horizonte";
		$state = "Ms";
		$zipCode = "30130-151";
		$country = "BR";
		$clientAddress = new Address($street, $number, $complement, $district, $city, $state, $zipCode, $country);

		$customer = new Customer();
        $customer->setId("cus_123213123");

		$bank = BankSlipTransaction::BANCO_DO_BRASIL;
		$value = "100.25";
		$documentNumber = "00000001";
		$transactionId = "0000001";
		$instructions = "Boleto de testes Teknisa";
		$daysToAddInBankSlipExpirationDate = "5";
		$bankSlipTransaction = new BankSlipTransaction($bank, $value, $documentNumber, $transactionId, $instructions, $daysToAddInBankSlipExpirationDate, $customer);

        $jsonResponse = '{
        	"id": "ch_1233454",
        	"last_transaction": {
        		"url": "https://urlteste.com"
        	},
        	"customer": {
        		"id": "cus_123213123"
        	}
        }';

		$response = array(
			"TransactionKey" => "ch_1233454",
			"BoletoUrl" => "https://urlteste.com"
		);

		$HTTPStatusCode = '200';

		$this->httpRequest->expects($this->once())
        			 ->method('setUrl');

        $this->httpRequest->expects($this->exactly(4))
        			 ->method('setOption');

		$this->httpRequest->expects($this->once())
        			 ->method('execute')
        			 ->willReturn($jsonResponse);

        $this->httpRequest->expects($this->once())
        			 ->method('getInfo')
        			 ->willReturn($HTTPStatusCode);

        $this->httpRequest->expects($this->once())
        			 ->method('close');

        $theResponse = $this->paymentImplMundiPagg->bankSlipTransaction($bankSlipTransaction);

        $this->assertEquals($theResponse, $response);
	}

	public function testBankSantander(){

		$street = "Rua Pernambuco";
		$number = "1000";
		$complement = "Cj 102";
		$district = "Funcionários";
		$city = "Belo Horizonte";
		$state = "Ms";
		$zipCode = "30130-151";
		$country = "BR";
		$clientAddress = new Address($street, $number, $complement, $district, $city, $state, $zipCode, $country);

		$customer = new Customer();
        $customer->setId("cus_123213123");

		$bank = BankSlipTransaction::SANTANDER;
		$value = "100.25";
		$documentNumber = "00000001";
		$transactionId = "0000001";
		$instructions = "Boleto de testes Teknisa";
		$daysToAddInBankSlipExpirationDate = "5";
		$bankSlipTransaction = new BankSlipTransaction($bank, $value, $documentNumber, $transactionId, $instructions, $daysToAddInBankSlipExpirationDate, $customer);

        $jsonResponse = '{
        	"id": "ch_1233454",
        	"last_transaction": {
        		"url": "https://urlteste.com"
        	},
        	"customer": {
        		"id": "cus_123213123"
        	}
        }';

		$response = array(
			"TransactionKey" => "ch_1233454",
			"BoletoUrl" => "https://urlteste.com"
		);

		$HTTPStatusCode = '200';

		$this->httpRequest->expects($this->once())
        			 ->method('setUrl');

        $this->httpRequest->expects($this->exactly(4))
        			 ->method('setOption');

		$this->httpRequest->expects($this->once())
        			 ->method('execute')
        			 ->willReturn($jsonResponse);

        $this->httpRequest->expects($this->once())
        			 ->method('getInfo')
        			 ->willReturn($HTTPStatusCode);

        $this->httpRequest->expects($this->once())
        			 ->method('close');

        $theResponse = $this->paymentImplMundiPagg->bankSlipTransaction($bankSlipTransaction);

        $this->assertEquals($theResponse, $response);
	}

	public function testBankBradesco(){

		$street = "Rua Pernambuco";
		$number = "1000";
		$complement = "Cj 102";
		$district = "Funcionários";
		$city = "Belo Horizonte";
		$state = "Ms";
		$zipCode = "30130-151";
		$country = "BR";
		$clientAddress = new Address($street, $number, $complement, $district, $city, $state, $zipCode, $country);

		$customer = new Customer();
        $customer->setId("cus_123213123");

		$bank = BankSlipTransaction::BRADESCO;
		$value = "100.25";
		$documentNumber = "00000001";
		$transactionId = "0000001";
		$instructions = "Boleto de testes Teknisa";
		$daysToAddInBankSlipExpirationDate = "5";
		$bankSlipTransaction = new BankSlipTransaction($bank, $value, $documentNumber, $transactionId, $instructions, $daysToAddInBankSlipExpirationDate, $customer);

        $jsonResponse = '{
        	"id": "ch_1233454",
        	"last_transaction": {
        		"url": "https://urlteste.com"
        	},
        	"customer": {
        		"id": "cus_123213123"
        	}
        }';

		$response = array(
			"TransactionKey" => "ch_1233454",
			"BoletoUrl" => "https://urlteste.com"
		);

		$HTTPStatusCode = '200';

		$this->httpRequest->expects($this->once())
        			 ->method('setUrl');

        $this->httpRequest->expects($this->exactly(4))
        			 ->method('setOption');

		$this->httpRequest->expects($this->once())
        			 ->method('execute')
        			 ->willReturn($jsonResponse);

        $this->httpRequest->expects($this->once())
        			 ->method('getInfo')
        			 ->willReturn($HTTPStatusCode);

        $this->httpRequest->expects($this->once())
        			 ->method('close');

        $theResponse = $this->paymentImplMundiPagg->bankSlipTransaction($bankSlipTransaction);

        $this->assertEquals($theResponse, $response);
	}

	public function testBankItau(){

		$street = "Rua Pernambuco";
		$number = "1000";
		$complement = "Cj 102";
		$district = "Funcionários";
		$city = "Belo Horizonte";
		$state = "Ms";
		$zipCode = "30130-151";
		$country = "BR";
		$clientAddress = new Address($street, $number, $complement, $district, $city, $state, $zipCode, $country);

		$customer = new Customer();
        $customer->setId("cus_123213123");

		$bank = BankSlipTransaction::ITAU;
		$value = "100.25";
		$documentNumber = "00000001";
		$transactionId = "0000001";
		$instructions = "Boleto de testes Teknisa";
		$daysToAddInBankSlipExpirationDate = "5";
		$bankSlipTransaction = new BankSlipTransaction($bank, $value, $documentNumber, $transactionId, $instructions, $daysToAddInBankSlipExpirationDate, $customer);

        $jsonResponse = '{
        	"id": "ch_1233454",
        	"last_transaction": {
        		"url": "https://urlteste.com"
        	},
        	"customer": {
        		"id": "cus_123213123"
        	}
        }';

		$response = array(
			"TransactionKey" => "ch_1233454",
			"BoletoUrl" => "https://urlteste.com"
		);

		$HTTPStatusCode = '200';

		$this->httpRequest->expects($this->once())
        			 ->method('setUrl');

        $this->httpRequest->expects($this->exactly(4))
        			 ->method('setOption');

		$this->httpRequest->expects($this->once())
        			 ->method('execute')
        			 ->willReturn($jsonResponse);

        $this->httpRequest->expects($this->once())
        			 ->method('getInfo')
        			 ->willReturn($HTTPStatusCode);

        $this->httpRequest->expects($this->once())
        			 ->method('close');

        $theResponse = $this->paymentImplMundiPagg->bankSlipTransaction($bankSlipTransaction);

        $this->assertEquals($theResponse, $response);
	}

	public function testBankHsbc(){

		$street = "Rua Pernambuco";
		$number = "1000";
		$complement = "Cj 102";
		$district = "Funcionários";
		$city = "Belo Horizonte";
		$state = "Ms";
		$zipCode = "30130-151";
		$country = "BR";
		$clientAddress = new Address($street, $number, $complement, $district, $city, $state, $zipCode, $country);

		$customer = new Customer();
        $customer->setId("cus_123213123");

		$bank = BankSlipTransaction::HSBC;
		$value = "100.25";
		$documentNumber = "00000001";
		$transactionId = "0000001";
		$instructions = "Boleto de testes Teknisa";
		$daysToAddInBankSlipExpirationDate = "5";
		$bankSlipTransaction = new BankSlipTransaction($bank, $value, $documentNumber, $transactionId, $instructions, $daysToAddInBankSlipExpirationDate, $customer);

        $jsonResponse = '{
        	"id": "ch_1233454",
        	"last_transaction": {
        		"url": "https://urlteste.com"
        	},
        	"customer": {
        		"id": "cus_123213123"
        	}
        }';

		$response = array(
			"TransactionKey" => "ch_1233454",
			"BoletoUrl" => "https://urlteste.com"
		);

		$HTTPStatusCode = '200';

		$this->httpRequest->expects($this->once())
        			 ->method('setUrl');

        $this->httpRequest->expects($this->exactly(4))
        			 ->method('setOption');

		$this->httpRequest->expects($this->once())
        			 ->method('execute')
        			 ->willReturn($jsonResponse);

        $this->httpRequest->expects($this->once())
        			 ->method('getInfo')
        			 ->willReturn($HTTPStatusCode);

        $this->httpRequest->expects($this->once())
        			 ->method('close');

        $theResponse = $this->paymentImplMundiPagg->bankSlipTransaction($bankSlipTransaction);

        $this->assertEquals($theResponse, $response);
	}

	public function testBankError(){
		$this->expectException(Exception::class);
        $this->expectExceptionMessage("Unsupported bank.");
        $this->expectExceptionCode(Exception::UNSUPPORTED_BANK);
		$street = "Rua Pernambuco";
		$number = "1000";
		$complement = "Cj 102";
		$district = "Funcionários";
		$city = "Belo Horizonte";
		$state = "Ms";
		$zipCode = "30130-151";
		$country = "BR";
		$clientAddress = new Address($street, $number, $complement, $district, $city, $state, $zipCode, $country);

		$customer = new Customer();
        $customer->setId("cus_123213123");

		$bank = "ERROR";
		$value = "100.25";
		$documentNumber = "00000001";
		$transactionId = "0000001";
		$instructions = "Boleto de testes Teknisa";
		$daysToAddInBankSlipExpirationDate = "5";
		$bankSlipTransaction = new BankSlipTransaction($bank, $value, $documentNumber, $transactionId, $instructions, $daysToAddInBankSlipExpirationDate, $customer);

        $this->paymentImplMundiPagg->bankSlipTransaction($bankSlipTransaction);
	}

	public function testRevoke(){
		
		$revoke = new Revoke("123", RequestURL::MUNDIPAGG_CUSTOMERS, "1000.00");
		$jsonResponse = '{
        	"id": "ch_1233454"
        }';

		$response = "ch_1233454";

		$HTTPStatusCode = '200';

		$this->httpRequest->expects($this->once())
        			 ->method('setUrl');

        $this->httpRequest->expects($this->exactly(4))
        			 ->method('setOption');

		$this->httpRequest->expects($this->once())
        			 ->method('execute')
        			 ->willReturn($jsonResponse);

        $this->httpRequest->expects($this->once())
        			 ->method('getInfo')
        			 ->willReturn($HTTPStatusCode);

        $this->httpRequest->expects($this->once())
        			 ->method('close');

        $theResponse = $this->paymentImplMundiPagg->revoke($revoke);

        $this->assertEquals($theResponse, $response);
	}

	public function testBaseURLCards(){
		
		$revoke = new Revoke("123", RequestURL::MUNDIPAGG_CARDS);
		$jsonResponse = '{
        	"id": "ch_1233454"
        }';

		$response = "ch_1233454";

		$HTTPStatusCode = '200';

		$this->httpRequest->expects($this->once())
        			 ->method('setUrl');

        $this->httpRequest->expects($this->exactly(4))
        			 ->method('setOption');

		$this->httpRequest->expects($this->once())
        			 ->method('execute')
        			 ->willReturn($jsonResponse);

        $this->httpRequest->expects($this->once())
        			 ->method('getInfo')
        			 ->willReturn($HTTPStatusCode);

        $this->httpRequest->expects($this->once())
        			 ->method('close');

        $theResponse = $this->paymentImplMundiPagg->revoke($revoke);

        $this->assertEquals($theResponse, $response);
	}

	public function testBaseURLCharges(){
		
		$revoke = new Revoke("123", RequestURL::MUNDIPAGG_CHARGES);
		$jsonResponse = '{
        	"id": "ch_1233454"
        }';

		$response = "ch_1233454";

		$HTTPStatusCode = '200';

		$this->httpRequest->expects($this->once())
        			 ->method('setUrl');

        $this->httpRequest->expects($this->exactly(4))
        			 ->method('setOption');

		$this->httpRequest->expects($this->once())
        			 ->method('execute')
        			 ->willReturn($jsonResponse);

        $this->httpRequest->expects($this->once())
        			 ->method('getInfo')
        			 ->willReturn($HTTPStatusCode);

        $this->httpRequest->expects($this->once())
        			 ->method('close');

        $theResponse = $this->paymentImplMundiPagg->revoke($revoke);

        $this->assertEquals($theResponse, $response);
	}

	public function testRevokeBaseUrlError(){
		$this->expectException(Exception::class);
        $this->expectExceptionMessage("Unsupported Base URL.");
        $this->expectExceptionCode(Exception::UNSUPPORTED_BASE_URL);
		
		$revoke = new Revoke("123", "error");

        $this->paymentImplMundiPagg->revoke($revoke);
	}

	public function testConsult(){
		
		$consult = new Consult("123", RequestURL::MUNDIPAGG_CUSTOMERS);
		$jsonResponse = '{
        	"id": "ch_1233454"
        }';

		$response = array(
			"id" => "ch_1233454"
		);

		$HTTPStatusCode = '200';

		$this->httpRequest->expects($this->once())
        			 ->method('setUrl');

        $this->httpRequest->expects($this->exactly(3))
        			 ->method('setOption');

		$this->httpRequest->expects($this->once())
        			 ->method('execute')
        			 ->willReturn($jsonResponse);

        $this->httpRequest->expects($this->once())
        			 ->method('getInfo')
        			 ->willReturn($HTTPStatusCode);

        $this->httpRequest->expects($this->once())
        			 ->method('close');

        $theResponse = $this->paymentImplMundiPagg->consult($consult);

        $this->assertEquals($theResponse, $response);
	}

	public function testFactory(){
		$factory = $this->paymentImplMundiPagg->factory("000000001");
		$this->assertInstanceOf(PaymentImpl::class, $factory);
	}
}