describe('JSONContainerLoader', function() {

	var thenable = function(data) {
		return {
			then: function(fn) {
				var dataReturned;
				if (fn) {
					dataReturned = fn(data);
				}
				return thenable(dataReturned || data);
			}
		};
	};
	var screenService = {
		i18n: function(text) {
			return text;
		}
	};
	var zhPromise = new ZHPromise(Q);
	var templateManager = {
		project: {},
		projectPromise: zhPromise.when(),
		getMappedPackages: function() {
			return {
				default: ["./"]
			};
		}
	};
	var restEngine = new RestEngine(console, function(p) {
		lastParams = p;
		return {
			success: function(fn) {
				if (fn) {
					fn(containerToLoad);
				}
				return thenable();
			}
		};
	}, templateManager, {}, new ZhFeatures(console), zhPromise, {
		onRequestStart: function() {},
		onRequestMetaDataStart: function() {}
	}, {});
	var jsonContainerLoader = new JSONContainerLoader(restEngine, zhPromise, screenService);

	var containerToLoad;

	beforeEach(function() {
		containerToLoad = {
			"id": 1003475497,
			"name": "login",
			"template": "container/window.html",
			"widgets": [{
				"id": 54545
			}]
		};
	});

	it("Method 'loadContainer': Should return a promise", function(done) {
		var fn = function(container) {
			expect(container.name).toEqual('login');
			done();
		};

		jsonContainerLoader.loadContainer('login').then(fn, fn);
	});

	it("Method 'loadContainer': Should return a reject promise if the name parameter is incorrect or doen't exist", function(done) {
		// This functions will be encadeated because i use a mocked promise in this case, to simplify the unit tests
		jsonContainerLoader.loadContainer().then(null, function(error) {
			expect(error.err).toEqual('Invalid parameters');
			jsonContainerLoader.loadContainer(null).then(null, function(error) {
				expect(error.err).toEqual('Invalid parameters');
				jsonContainerLoader.loadContainer({}).then(null, function(error) {
					expect(error.err).toEqual('Invalid parameters');
					jsonContainerLoader.loadContainer([]).then(null, function(error) {
						expect(error.err).toEqual('Invalid parameters');
						jsonContainerLoader.loadContainer(123).then(null, function(error) {
							expect(error.err).toEqual('Invalid parameters');
							done();
						});
					});
				});
			});
		});

	});

	it("Method 'loadContainer': If the container have not been loaded, the JSONContainerLoader class should load this container", function(done) {
		jsonContainerLoader.loadContainer('login').then(function(container) {
			expect(container).toEqual(containerToLoad);
			done();
		});
	});

});