describe("Query", function() {

	var query = new Query();

	it("Operator Like", function() {
		var query = new Query().where('name').like('Matheus');
		var where = query.where();
		expect(where[0].name).toEqual('name');
		expect(where[0].operator).toEqual('LIKE');
		expect(where[0].value).toEqual('Matheus');
	});

	it("Operator LikeInsensitive", function() {
		var query = new Query().where('name').likeInsensitive('Matheus');
		var where = query.where();
		expect(where[0].name).toEqual('name');
		expect(where[0].operator).toEqual('LIKE_I');
		expect(where[0].value).toEqual('Matheus');
	});

	it("Operator Not Like", function() {
		var query = new Query().where('name').notLike('Matheus');
		var where = query.where();
		expect(where[0].name).toEqual('name');
		expect(where[0].operator).toEqual('NOT_LIKE');
		expect(where[0].value).toEqual('Matheus');
	});

	it("Operator Like All", function() {
		var query = new Query().where('name').likeAll('Matheus');
		var where = query.where();
		expect(where[0].name).toEqual('name');
		expect(where[0].operator).toEqual('LIKE_ALL');
		expect(where[0].value).toEqual('Matheus');
	});

	it("Operator In_Array", function() {
		var query = new Query().where('name').in([1, 2, 3]);
		var where = query.where();
		expect(where[0].name).toEqual('name');
		expect(where[0].operator).toEqual('IN');
		expect(where[0].value).toEqual([1, 2, 3]);
	});

	it("Operator Not In_Array", function() {
		var query = new Query().where('name').notIn([1, 2, 3]);
		var where = query.where();
		expect(where[0].name).toEqual('name');
		expect(where[0].operator).toEqual('NOT_IN');
		expect(where[0].value).toEqual([1, 2, 3]);
	});

	it("Operator In_String", function() {
		var query = new Query().where('name').in('1,2,3');
		var where = query.where();
		expect(where[0].name).toEqual('name');
		expect(where[0].operator).toEqual('IN');
		expect(where[0].value).toEqual('1,2,3');
	});

	it("Operator Not In_String", function() {
		var query = new Query().where('name').notIn('1,2,3');
		var where = query.where();
		expect(where[0].name).toEqual('name');
		expect(where[0].operator).toEqual('NOT_IN');
		expect(where[0].value).toEqual('1,2,3');
	});

	it("Operator Between_Array", function() {
		var query = new Query().where('salary').between([2000, 3000]);
		var where = query.where();
		expect(where[0].name).toEqual('salary');
		expect(where[0].operator).toEqual('BETWEEN');
		expect(where[0].value).toEqual([2000, 3000]);
	});

	it("Operator Not Between_Array", function() {
		var query = new Query().where('salary').notBetween([2000, 3000]);
		var where = query.where();
		expect(where[0].name).toEqual('salary');
		expect(where[0].operator).toEqual('NOT_BETWEEN');
		expect(where[0].value).toEqual([2000, 3000]);
	});

	it("Operator Between_String", function() {
		var query = new Query().where('salary').between('2000, 3000');
		var where = query.where();
		expect(where[0].name).toEqual('salary');
		expect(where[0].operator).toEqual('BETWEEN');
		expect(where[0].value).toEqual('2000, 3000');
	});

	it("Operator Not Between_String", function() {
		var query = new Query().where('salary').notBetween('2000, 3000');
		var where = query.where();
		expect(where[0].name).toEqual('salary');
		expect(where[0].operator).toEqual('NOT_BETWEEN');
		expect(where[0].value).toEqual('2000, 3000');
	});

	it("Operator Equals", function() {
		var query = new Query().where('name').equals('Matheus');
		var where = query.where();
		expect(where[0].name).toEqual('name');
		expect(where[0].operator).toEqual('=');
		expect(where[0].value).toEqual('Matheus');
	});

	it("Operator Not Equals", function() {
		var query = new Query().where('name').notEquals('Matheus');
		var where = query.where();
		expect(where[0].name).toEqual('name');
		expect(where[0].operator).toEqual('<>');
		expect(where[0].value).toEqual('Matheus');
	});

	it("Operator Greater", function() {
		var query = new Query().where('salary').greater(2000);
		var where = query.where();
		expect(where[0].name).toEqual('salary');
		expect(where[0].operator).toEqual('>');
		expect(where[0].value).toEqual(2000);
	});

	it("Operator Greater or Equal", function() {
		var query = new Query().where('salary').greaterOrEqual(2000);
		var where = query.where();
		expect(where[0].name).toEqual('salary');
		expect(where[0].operator).toEqual('>=');
		expect(where[0].value).toEqual(2000);
	});

	it("Operator Lower", function() {
		var query = new Query().where('salary').lower(2000);
		var where = query.where();
		expect(where[0].name).toEqual('salary');
		expect(where[0].operator).toEqual('<');
		expect(where[0].value).toEqual(2000);
	});

	it("Operator Lower or Equal", function() {
		var query = new Query().where('salary').lowerOrEqual(2000);
		var where = query.where();
		expect(where[0].name).toEqual('salary');
		expect(where[0].operator).toEqual('<=');
		expect(where[0].value).toEqual(2000);
	});

	it("Limit", function() {
		var callLimitTwice = function() {
			var query = new Query().where('salary').lower(2000).limit(2000, 1400).limit(300);
		};

		var callLimitOnce = function() {
			var query = new Query().where('salary').lower(2000).limit(2000, 1400);
		};

		expect(callLimitTwice).toThrow();
		expect(callLimitOnce).not.toThrow();
	});

	it("Offset", function() {
		var callOffsetTwice = function() {
			var query = new Query().where('salary').lower(2000).offset(2000, 1400).offset(300);
		};

		var callOffsetOnce = function() {
			var query = new Query().where('salary').lower(2000).offset(2000, 1400);
		};

		expect(callOffsetTwice).toThrow();
		expect(callOffsetOnce).not.toThrow();
	});

	it("Mixed", function() {
		var query = new Query().where('name').equals('blabla').where('salary').lower(2000).offset(
			2000);
		var where = query.where();

		expect(where[0].name).toEqual('name');
		expect(where[0].operator).toEqual('=');
		expect(where[0].value).toEqual('blabla');

		expect(where[1].name).toEqual('salary');
		expect(where[1].operator).toEqual('<');
		expect(where[1].value).toEqual(2000);
	});

	it("Operator Is Null", function() {
		var query = new Query().where('name').equals('Teste').where('value').isNull();
		var where = query.where();

		expect(where[0].name).toEqual('name');
		expect(where[0].operator).toEqual('=');
		expect(where[0].value).toEqual('Teste');
		expect(where.length).toEqual(2);
		expect(where[1].name).toEqual('value');
		expect(where[1].operator).toEqual('IS_NULL');
		expect(where[1].value).toEqual('');
	});

	it("Operator Is Not Null", function() {
		var query = new Query().where('name').equals('Teste').where('value').isNotNull();
		var where = query.where();

		expect(where[0].name).toEqual('name');
		expect(where[0].operator).toEqual('=');
		expect(where[0].value).toEqual('Teste');
		expect(where.length).toEqual(2);
		expect(where[1].name).toEqual('value');
		expect(where[1].operator).toEqual('IS_NOT_NULL');
		expect(where[1].value).toEqual('');
	});

	it("Order by with one column", function() {
		var query = new Query().orderBy('name');
		var where = query.where();
		expect(where[0].name).toEqual([{name: 'name', direction: 'ASC'}]);
		expect(where[0].operator).toEqual('ORDER_BY');
	});

	it("Order by with one column and direction", function() {
		var query = new Query().orderBy('name', 'ASC');
		var where = query.where();
		expect(where[0].name).toEqual([{name: 'name', direction: 'ASC'}]);
		expect(where[0].operator).toEqual('ORDER_BY');
	});

	it("Order by with multiple columns", function() {
		var query = new Query().orderBy(['name', 'description']);
		var where = query.where();
		expect(where[0].name).toEqual([{name: 'name', direction: 'ASC'}, {name: 'description', direction: 'ASC'}]);
		expect(where[0].operator).toEqual('ORDER_BY');
	});

	it("Order by with multiple columns and custom direction", function() {
		var query = new Query().orderBy([{name: 'name', direction: 'DESC'}, 'description']);
		var where = query.where();
		expect(where[0].name).toEqual([{name: 'name', direction: 'DESC'}, {name: 'description', direction: 'ASC'}]);
		expect(where[0].operator).toEqual('ORDER_BY');
	});

	it("Group by with one column", function() {
		var query = new Query().groupBy('name');
		var where = query.where();
		expect(where[0].name).toEqual(['name']);
		expect(where[0].operator).toEqual('GROUP_BY');
	});

	it("Group by with multiple columns", function() {
		var query = new Query().groupBy(['name', 'description']);
		var where = query.where();
		expect(where[0].name).toEqual(['name', 'description']);
		expect(where[0].operator).toEqual('GROUP_BY');
	});

	it("Build query", function() {
		var where = new Query().buildQuery({ name: "Matheus" },"NOT_LIKE");
		expect(where[0].name).toEqual('name');
		expect(where[0].operator).toEqual('NOT_LIKE');
		expect(where[0].value).toEqual('Matheus');
	});	

	it("Where query", function() {
		var where = new Query().where({ name: "Matheus" }).where();
		expect(where[0].name).toEqual('name');
		expect(where[0].operator).toEqual('=');
		expect(where[0].value).toEqual('Matheus');
	});	
});