describe("RestEngine", function () {
	var restEngine;
	var $httpBackend;
	var $rootScope;
	var zhPromise;

	var buildMock = function () {
		var mock = {
			whenSuccess: function () {
				console.log("success\n");
			},
			whenError: function () {
				console.log("error\n");
			}
		};
		spyOn(mock, 'whenSuccess').and.callThrough();
		spyOn(mock, 'whenError').and.callThrough();
		return mock;
	};

	beforeEach(function () {
		window.templateUrl = "../../dyman/templates/dyman/";
		window.metadataUrl = "/json_test/test/";
		window.metadataUrls = ["/json_test2/test/"];
		window.serviceUrl = "../backend/service/index.php";

		zhPromise = new ZHPromise(Q);

		module('ZeedhiApp');
		inject(function (_restEngine_) {
			restEngine = _restEngine_;
		});
	});

	afterEach(function () {
		delete window.metadataUrls;
	});

	beforeEach(inject(function (_$rootScope_, _$httpBackend_) {
		_$httpBackend_.when("POST", '../backend/service/index.php/success/find').respond({ "data": [] }, {});
		_$httpBackend_.when("POST", '../backend/service/index.php/error/find').respond({ "error": "deu erro" }, {});

		_$httpBackend_.when("GET", '/json_test/test/success/find').respond({ "data": {} }, {});
		_$httpBackend_.when("GET", '/json_test/test/error/find').respond(500, { "error": { message: "Unexpected token" } }, {});
		_$httpBackend_.when("GET", '/json_test2/test/error/find').respond(500, { "error": { message: "Unexpected token" } }, {});
		$rootScope = _$rootScope_;
		$httpBackend = _$httpBackend_;
	}));

	it("If success the then callback must be call", function (done) {
		var mock = buildMock();
		console.error = function () { };
		ZhHelper.doAsyncTest(function (_done) {
			restEngine.request("/success/find").then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function () {
			ZhHelper.securityApply($rootScope);
			Util.doAsync(function () {
				ZhHelper.securityApply($rootScope);
				Util.doAsync(function () {
					ZhHelper.securityApply($rootScope);
					expect(mock.whenSuccess).toHaveBeenCalled();
					expect(mock.whenError).not.toHaveBeenCalled();
					done();
				});
			});
		}, 1000, $rootScope, $httpBackend);
	});

	it("If error the then callback error must be call", function (done) {
		var mock = buildMock();
		ZhHelper.doAsyncTest(function (_done) {
			restEngine.request("/error/find").then(mock.whenSuccess, mock.whenError);
			ZhHelper.securityApply($rootScope);
		}, function () {
			ZhHelper.securityApply($rootScope);
			Util.doAsync(function () {
				ZhHelper.securityApply($rootScope);
				Util.doAsync(function () {
					ZhHelper.securityApply($rootScope);
					expect(mock.whenSuccess).not.toHaveBeenCalled();
					expect(mock.whenError).toHaveBeenCalled();
					done();
				});
			});
		}, 1000, $rootScope, $httpBackend);
	});

	it("If metadata success the then callback must be call", function (done) {
		var mock = buildMock();
		restEngine.requestMetaData("success/find", {})
			.then(mock.whenSuccess)
			.catch(mock.whenError)
			.then(function () {
				expect(mock.whenSuccess).toHaveBeenCalled();
				expect(mock.whenError).not.toHaveBeenCalled();
				done();
			});
		$httpBackend.flush();

	});

	it("If metadata error the then callback error must be call", function (done) {
		var mock = buildMock();
		restEngine.requestMetaData("error/find", {})
			.then(mock.whenSuccess)
			.catch(mock.whenError)
			.then(function () {
				expect(mock.whenSuccess).not.toHaveBeenCalled();
				expect(mock.whenError).toHaveBeenCalled();
				done();
			});
		$httpBackend.flush();
	});

	it("If the first metadata source return an error, the second must be called", function (done) {
		$httpBackend.expectGET('/json_test/test/errorFirstCall/find').respond(500, { "error": "deu erro" }, {});
		$httpBackend.expectGET('/json_test2/test/errorFirstCall/find').respond(200, { "data": {} }, {});

		var mock = buildMock();
		restEngine.requestMetaData("errorFirstCall/find", {})
			.then(mock.whenSuccess)
			.catch(mock.whenError)
			.then(function () {
				expect(mock.whenSuccess).toHaveBeenCalled();
				expect(mock.whenError).not.toHaveBeenCalled();
				done();
			});
		$httpBackend.flush();
	});

	describe("Request parameters tests", function () {
		var eventAggregator;
		var USER_ID_PROPERTY = "USER_ID";
		var previousLocalStorageUserId = localStorage.getItem(USER_ID_PROPERTY);
		var calledRequestConfig;
		var templateManager;

		var httpMock = {
			http: function (requestConfig) {
				calledRequestConfig = requestConfig;
				return {
					success: function (callBack) { callBack({}); return this; },
					then: function (callBack) {
						if (callBack) {
							callBack({});
						}
						return this;
					}
				};
			}
		};

		beforeEach(function () {
			spyOn(httpMock, 'http').and.callThrough();
			inject(function (ZhLogger, _templateManager_, ScreenService, ZhFeatures, ZHPromise, _eventAggregator_, requestInterceptingEngine) {
				eventAggregator = _eventAggregator_;
				templateManager = _templateManager_;
				restEngine = new RestEngine(ZhLogger, httpMock.http, templateManager, ScreenService, ZhFeatures, ZHPromise, _eventAggregator_, requestInterceptingEngine);
			});
		});

		afterAll(function () {
			localStorage.setItem(USER_ID_PROPERTY, previousLocalStorageUserId);
		});

		it("request: should add properties to the request if there is a change via EventAggregator", function (done) {
			eventAggregator.subscribe("EventOnRequestStart", function (eventData) {
				eventData.requestConfig.headers.test = "someHeaderToTest";
			});
			localStorage.setItem(USER_ID_PROPERTY, "123");

			restEngine.request("http://someService.com/something", null, function () {
				expect(httpMock.http).toHaveBeenCalledTimes(1);
				expect(calledRequestConfig).toBeDefined();
				expect(calledRequestConfig.headers).toBeDefined();
				expect(calledRequestConfig.headers.test).toBeDefined();
				expect(calledRequestConfig.headers.test).toEqual("someHeaderToTest");
				done();
			});
		});

		it("EventOnRequestIdle should be fired when property onRequestIdleTimeout exists", function (done) {
			spyOn(eventAggregator, 'onRequestIdle').and.callThrough();
			inject(function (ZhLogger, _templateManager_, ScreenService, ZhFeatures, ZHPromise, _eventAggregator_, requestInterceptingEngine) {
				eventAggregator = _eventAggregator_;
				templateManager = _templateManager_;
				templateManager.project.onRequestIdleTimeout = 600;
				templateManager.projectPromise = zhPromise.when();
				restEngine = new RestEngine(ZhLogger, httpMock.http, templateManager, ScreenService, ZhFeatures, ZHPromise, _eventAggregator_, requestInterceptingEngine);
			});
			var eventCallback = function (eventData) {
				expect(eventAggregator.onRequestIdle).toHaveBeenCalled();
				done();
			};
			eventAggregator.subscribe("EventOnRequestIdle", eventCallback);
			templateManager.projectPromise.then(function(){
				restEngine.request("http://someService.com/something",null,function(){
					delete templateManager.project.onRequestIdleTimeout;
				});
			});
		});

		it("EventOnRequestIdle should be fired when property onRequestIdleTimeout exists and OnRequestIdleEvent_LastRequest too", function (done) {
			spyOn(eventAggregator, 'onRequestIdle').and.callThrough();
			inject(function (ZhLogger, _templateManager_, ScreenService, ZhFeatures, ZHPromise, _eventAggregator_, requestInterceptingEngine) {
				eventAggregator = _eventAggregator_;
				templateManager = _templateManager_;
				templateManager.project.onRequestIdleTimeout = 600;
				templateManager.projectPromise = zhPromise.when();
				restEngine = new RestEngine(ZhLogger, httpMock.http, templateManager, ScreenService, ZhFeatures, ZHPromise, _eventAggregator_, requestInterceptingEngine);
			});
			var eventCallback = function (eventData) {
				expect(eventAggregator.onRequestIdle).toHaveBeenCalled();
				done();
			};
			eventAggregator.subscribe("EventOnRequestIdle", eventCallback);
			localStorage.setItem('OnRequestIdleEvent_LastRequest', Date.now());
			templateManager.projectPromise.then(function(){
				restEngine.request("http://someService.com/something",null,function(){
					delete templateManager.project.onRequestIdleTimeout;
					localStorage.removeItem('OnRequestIdleEvent_LastRequest');
				});
			});
		});

	});
});