function CreditCardController(ScreenService, templateManager, LoginRepository, CreditCardService, ConsumerRepository){

    this.reloadCreditCardForm = function(widget) {
        widget.currentRow = {};
        widget.currentRow.SETDEFAULT = 'P';
        ConsumerRepository.findOne().then(function(consumer) {
            CreditCardService.getAvaliableFlags(consumer['CDFILIAL']).then(function(avaliableFlags) {
                widget.fields[4].dataSource.data = decodeFlags(avaliableFlags.dataset.FlagsId);
                widget.widgets[0].dataSource.data = widget.fields[4].dataSource.data;
            }).catch(function(err) {
                ScreenService.errorNotification(err.data.error.split("Exception: ")[1]);
            });
        }).catch(function() {
            ScreenService.errorNotification("Usuário não encontrado!");
        });
    };

    this.getCreditCard = function(widget) {
        ConsumerRepository.findOne().then(function(consumer){
            CreditCardService.getCreditCard(consumer).then(function(creditCards){
                creditCards.dataset.CreditCards = formatDataSource(creditCards.dataset.CreditCards);
                widget.dataSource.data = creditCards.dataset.CreditCards;
                widget.reload();
            }).catch(function() {
                ScreenService.errorNotification("Não foi possível buscar seus cartões!");
            })
        }).catch(function() {
            ScreenService.errorNotification("Usuário não encontrado!");
        });
    };

    this.register = function(widget) {
        if(!verifyRequiredFields(widget.currentRow)) {
            ScreenService.errorNotification("Preencha todos os campos!");
        } else if(!verifyCreditCardExpirationDate(widget.currentRow)) {
            ScreenService.errorNotification("Cartão com data expirada!");
        } else if(!verifyCreditCardFlag(widget.currentRow)) {
            widget.widgets[0].fields[0].dataSource = widget.fields[4].dataSource;
            ScreenService.openPopup(widget.widgets[0]);
        } else {
            registerValidCreditCard(widget.currentRow);
        }
    };

    this.deleteCheckedRows = function(widget) {
        ScreenService.confirmMessage("Você realmente deseja excluir?", "question")
        .then(function() {
            widget.getCheckedRows().forEach(function(row) {
                row.__rightActions = [];
            });
            var checkedRows = JSON.stringify(widget.getCheckedRows());
            CreditCardService.deleteCreditCard(checkedRows)
            .then(function() {
                ScreenService.successNotification("Cartão excluído com sucesso!");
                ScreenService.openWindow('creditCardList');
            })
            .catch(function(err) {
                var error = err.data.error.split("Exception: ")[1];
                ScreenService.errorNotification(error);
            });
        });
    }

    this.setCreditCardAsDefault = function(row) {
        CreditCardService.setCreditCardAsDefault(row)
        .then(function() {
            ScreenService.successNotification("Cartão principal alterado!");
            ScreenService.openWindow('creditCardList');
        })
        .catch(function(err) {
            var error = err.data.error.split("Exception: ")[1];
            ScreenService.errorNotification(error);
        });
    }

    function decodeFlags(flagsId){
        flagsId.forEach(function(flagId){
            if (flagId.IDESITEF == 1){
                flagId.value = "Visa";
            }
            else if (flagId.IDESITEF == 2){
                flagId.value = "Mastercard";
            }
            else if (flagId.IDESITEF == 3){
                flagId.value = "Amex";
            }
            else if (flagId.IDESITEF == 5){
                flagId.value = "Hipercard";
            }
            else if (flagId.IDESITEF == 6){
                flagId.value = "Aura";
            }
            else if (flagId.IDESITEF == 33){
                flagId.value = "Diners";
            }
            else if (flagId.IDESITEF == 41){
                flagId.value = "Elo";
            }
            else if (flagId.IDESITEF == 224){
                flagId.value = "Alelo";
            }
            else if (flagId.IDESITEF == 281){
                flagId.value = "Sodexo";
            }
            else {
                flagId.value = "Bandeira não encontrada";
            }
        });
        return flagsId;
    }

    function registerValidCreditCard(row) {
        ConsumerRepository.findOne().then(function(consumer){
            row.CD_CLIENTE     = consumer['CDCLIENTE'];
            row.CD_CONSUMIDOR  = consumer['CDCONSUMIDOR'];
            row.CD_FILIAL      = consumer['CDFILIAL'];
            row.cardNumber     = row.cardNumber.replace(/ /g, "");
            var expirationDate = formatDateToMonthYear(row.EXPIRATION_DATE);
            row.EXP_MONTH      = expirationDate.month;
            row.EXP_YEAR       = expirationDate.year;
            persistCreditCardRegister(row);
        })
        .catch(function (){
            ScreenService.errorNotification("Usuário não encontrado!");
        });
    }

    function persistCreditCardRegister(row){
        CreditCardService.registerCreditCard(row).then(
            function (result){
                ScreenService.successNotification("Cartão criado com sucesso!");
                ScreenService.openWindow('creditCardList');
            },
            function (){
                ScreenService.errorNotification("Ocorreu um erro a processar a operação. Tente novamente mais tarde.");
            }
        );
    }

    function formatDateToMonthYear(creditCardExpirationDate) {
        var expDate = creditCardExpirationDate.split("/");
        return {
            month: expDate[0],
            year: expDate[1]
        }
    }

    function formatDataSource(creditCards) {
        creditCards.forEach(function(creditCard) {
            creditCard.DTEXPIRACAO    = creditCard.DTEXPIRACAO.split("/");
            creditCard.DTEXPIRACAO[2] = creditCard.DTEXPIRACAO[2].replace("00", '');
            creditCard.DTEXPIRACAO[2] = creditCard.DTEXPIRACAO[2].split(' ')[0];
            creditCard.DTEXPIRACAO    = creditCard.DTEXPIRACAO[1]+"/"+creditCard.DTEXPIRACAO[2];
            creditCard.NRULTNUM       = padZeroes(creditCard.NRULTNUM, 4);
        });
        return creditCards;
    }

    function padZeroes(number, length) {
        var my_string = number;
        while (my_string.length < length) {
            my_string = "0"+my_string;
        }
        return my_string;
    }

    function verifyRequiredFields(row) {
         return row.cardNumber && row.NAME && !row.EXPIRATION_DATE == "" && row.CVV;
    }
    
    function verifyCreditCardExpirationDate(row) {
        var currentYear     = moment().format("YY");
        var currentMonth    = moment().format("MM");
        var expirationDate  = formatDateToMonthYear(row.EXPIRATION_DATE);
        var creditCardMonth = expirationDate.month;
        var creditCardYear  = expirationDate.year;
        return currentYear < creditCardYear || (currentYear === creditCardYear && currentMonth <= creditCardMonth);
    }
    
    function verifyCreditCardFlag(row) {
        return row.cardFlag;
    }


    // The cards array must have objects with bins in descendent order.
    var cards = [
        {
            flag: "default",
            authorizerId: "",
            path: "images/cards/default.png",
            bins: []
        },
        {
            flag: "elo",
            authorizerId: "41",
            path: "images/cards/elo.png",
            bins: ['650485', '650539','636368','438935','504175','451416',
                   '509048', '509067','509049','509069','509050','509074',
                   '509068', '509040', '509045','509051','509046','509066',
                   '509047', '509042', '509052','509043','509064','509040',
                   '36297','5067','4576','4011']
        },
        {
            flag: "diners",
            authorizerId: "33",
            path: "images/cards/diners.png",
            bins: ['301','305','36','38']
        },
        {
            flag: "americanExpress",
            authorizerId: "3",
            path: "images/cards/american_express.png",
            bins: ['34', '37']
        },
        {
            flag: "mastercard",
            authorizerId: "2",
            path: "images/cards/mastercard.png",
            bins: ['5']
        },
        {
            flag: "visa",
            authorizerId: "1",
            path: "images/cards/visa.png",
            bins: ['4']
        },
        {
            flag: "cabal",
            authorizerId: "12",
            path: "images/cards/cabal.png",
            bins: []
        },
        {
            flag: "mastercardPrivateLabel",
            authorizerId: "202",
            path: "images/cards/mastercard.png",
            bins: []
        },
        {
            flag: "aleloCultura",
            authorizerId: "223",
            path: "images/cards/elo.png",
            bins: []
        },
        {
            flag: "aleloRefeição",
            authorizerId: "224",
            path: "images/cards/elo.png",
            bins: []
        },
        {
            flag: "aleloAlimentação",
            authorizerId: "225",
            path: "images/cards/elo.png",
            bins: []
        },
        {
            flag: "sodexoValeCultura",
            authorizerId: "279",
            path: "images/cards/elo.png",
            bins: []
        },
        {
            flag: "sodexoValeRefeicao",
            authorizerId: "281",
            path: "images/cards/elo.png",
            bins: []
        },
        {
            flag: "sodexoGift",
            authorizerId: "282",
            path: "images/cards/elo.png",
            bins: []
        },
        {
            flag: "sodexoPremium",
            authorizerId: "283",
            path: "images/cards/elo.png",
            bins: []
        },
        {
            flag: "sodexoCombustivel",
            authorizerId: "284",
            path: "images/cards/elo.png",
            bins: []
        }
    ];

    this.checkCreditCardFlag = function(cardNumber, cardImageField) {
        // Remove all spaces from card number
        var cardNumber = cardNumber.replace(/\s/g, '');

        var card = this.getFlag(this.findCardFlag(cardNumber));
        cardImageField.source = card.path;
        cardImageField.setValue(card.authorizerId);
    };

    this.findCardFlag = function(cardNumber) {
        for(var cardCounter = 0; cardCounter < cards.length; cardCounter++) {
            for(var binCounter = 0; binCounter < cards[cardCounter].bins.length; binCounter++) {
                if (this.startsWith(cardNumber, cards[cardCounter].bins[binCounter]))
                    return cards[cardCounter].flag;
            }
        }
    };

    this.startsWith = function(str, prefix) {
        if (str.substr(0,prefix.length) == prefix)
            return true;
        else
            return false;
    };

    // Find the card flag through the flag name
    this.getFlag = function(flagName) {
        // Yes, I tried to use forEach instead of for but didn't work out.
        for (var i = 0; i < cards.length; i++) {
            if (cards[i].flag === flagName) {
                return cards[i];
            }
        }
    };

    this.getUserCards = function(cardsField){
        LoginRepository.findOne()
        .then(function(consumer) {
            CreditCardService.getCreditCard(consumer)
            .then(function(creditCards) {
                /*if(creditCards.dataset.IdUseCvv.IDUTCVVCARTARM === 'S'){
                    CreditCardService.setUseCvv(true);
                } else{
                    CreditCardService.setUseCvv(false);
                }*/
                if(!creditCards.dataset.CreditCards.length) {
                    return ScreenService.openWindow('creditCard');
                }
                cardsField.dataSource.data = [];
                creditCards.dataset.CreditCards.forEach(function(creditCard) {
                    creditCard.id = creditCard.CDCARTCRED;
                    creditCard.title = "Final " + padZeroes(creditCard.NRULTNUM, 4);
                    creditCard.image = "images/cards/" + creditCard.NMBANDEIRA.toLowerCase() + ".png";
                    cardsField.dataSource.data.push(creditCard);
                });
            })
            .catch(function() {
                ScreenService.errorNotification("Não foi possível buscar seus cartões!");
            })
        })
        .catch(function() {
            ScreenService.errorNotification("Usuário não encontrado!");
        });
    }

};

Configuration(function(ContextRegister){
    ContextRegister.register('CreditCardController', CreditCardController);
});
